// Copyright (C) 2022 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:wyatt_cli_toolbox/src/prompt/framework/framework.dart';
import 'package:wyatt_cli_toolbox/src/prompt/framework/theme.dart';

class Input extends Widget<String> {
  final String message;
  final String initialValue;
  final String? defaultValue;
  final bool Function(String)? validator;
  final Theme theme;

  Input({
    required this.message,
    this.initialValue = '',
    this.defaultValue,
    this.validator,
  }) : theme = Theme.defaultTheme;

  Input.withTheme({
    required this.message,
    required this.theme,
    this.initialValue = '',
    this.defaultValue,
    this.validator,
  });

  String promptInput({
    required String message,
    String? hint,
  }) {
    final buffer = StringBuffer()
      ..write(theme.inputPrefix)
      ..write(theme.messageStyle(message));
    if (hint != null) {
      buffer
        ..write(' ')
        ..write(theme.hintStyle(hint));
    }
    buffer
      ..write(theme.inputSuffix)
      ..write(' ');
    return buffer.toString();
  }

  String promptError({
    required String message,
  }) {
    final buffer = StringBuffer()
      ..write(theme.errorPrefix)
      ..write(theme.errorStyle(message));

    return buffer.toString();
  }

  @override
  State<Input> createState() => _InputState();
}

class _InputState extends State<Input> {
  String? _value;
  String? _error;

  @override
  void initState() {
    super.initState();
    _value = widget.initialValue;
  }

  @override
  void dispose() {
    if (_value != null) {
      // TODO(hpcl): print success
    }
    super.dispose();
  }

  @override
  void build() {
    if (_error != null) {
      context.writeln(widget.promptError(message: _error!));
    }
  }

  @override
  String prompt() {
    while (true) {
      context.write(
        widget.promptInput(
          message: widget.message,
          hint: widget.defaultValue,
        ),
      );
      final input = context.readLine(initialValue: widget.initialValue);
      final line = input.isEmpty && widget.defaultValue != null
          ? widget.defaultValue!
          : input;
      if (widget.validator != null) {
        try {
          widget.validator!(line);
        } on ValidationError catch (e) {
          setState(() {
            _error = e.message;
          });
          continue;
        }
      }
      setState(() {
        _value = line;
      });
      return _value!;
    }
  }
}

class ValidationError implements Exception {
  final String message;

  ValidationError(this.message);
}
